==> 1.0.2-dev.tar.gz: lib/src/route/route_dialog.dart <==
part of '../app_route.dart';

/// Callback when an action is performed, must return the result that will be dispatched to [OnActionResult]
typedef OnAction<T> = FutureOr<T> Function(BuildContext context);

/// Handle [Result] of [OnAction]
typedef OnActionResult<T> = void Function(BuildContext context, Result<T> value);

/// Dialog route builder
typedef DialogRouteBuilder = Future Function(BuildContext context, RouteSettings settings);

/// [AppRoute] of a Dialog
class DialogRoute<T> extends AppRoute<T> {
  final DialogRouteBuilder builder;

  DialogRoute.builder(
    Object name, {
    OnAction<T>? onDismiss,
    OnActionResult<T>? onDismissResult = popResult,
    bool barrierDismissible = true,
    Color? barrierColor = Colors.black54,
    String? barrierLabel,
    bool useSafeArea = true,
    bool useRootNavigator = true,
    required RouteWidgetBuilder builder,
  })  : builder = ((context, settings) => showDialog(
            context: context,
            barrierDismissible: barrierDismissible,
            barrierColor: barrierColor,
            barrierLabel: barrierLabel,
            useSafeArea: useSafeArea,
            useRootNavigator: useRootNavigator,
            routeSettings: settings,
            builder: (context) => wrapIfNotNull<Widget, OnAction<T>>(
                  item: onDismiss,
                  wrap: (child, onDismiss) => WillPopScope(
                      onWillPop: () => Result.wrapAsync(() => onDismiss(context)).then((value) {
                            onDismissResult?.call(context, value);
                            return false;
                          }),
                      child: child),
                  child: builder(context, settings.arguments as RouteArgs),
                ))),
        super(name.toString());

  DialogRoute.alertBuilder(
    Object name, {
    Object? title,
    EdgeInsetsGeometry? titlePadding,
    OnAction<T>? onDismiss,
    OnActionResult<T>? onDismissResult = popResult,
    AlertDialogAction<T>? positiveAction,
    AlertDialogAction<T>? negativeAction,
    AlertDialogAction<T>? neutralAction,
    EdgeInsetsGeometry contentPadding = const EdgeInsets.fromLTRB(24.0, 20.0, 24.0, 24.0),
    Color? backgroundColor,
    double? elevation,
    EdgeInsets insetPadding = const EdgeInsets.symmetric(horizontal: 40.0, vertical: 24.0),
    Clip clipBehavior = Clip.none,
    ShapeBorder? shape,
    bool scrollable = false,
    bool barrierDismissible = true,
    Color? barrierColor = Colors.black54,
    String? barrierLabel,
    bool useSafeArea = true,
    bool useRootNavigator = true,
    required RouteWidgetBuilder builder,
  }) : this.builder(name,
            onDismiss: onDismiss,
            onDismissResult: onDismissResult,
            barrierDismissible: barrierDismissible,
            barrierColor: barrierColor,
            barrierLabel: barrierLabel,
            useSafeArea: useSafeArea,
            useRootNavigator: useRootNavigator,
            builder: (context, args) => AlertDialog(
                title: title == null
                    ? null
                    : title is Widget
                        ? title
                        : Text(title.toString()),
                titlePadding: titlePadding,
                content: builder(context, args),
                contentPadding: contentPadding,
                actions: [
                  neutralAction?._withAction(_ActionId.neutral),
                  negativeAction?._withAction(_ActionId.negative),
                  positiveAction?._withAction(_ActionId.positive)
                ].whereNotNull().map((entry) => entry._build(context)).toList(growable: false),
                backgroundColor: backgroundColor,
                elevation: elevation,
                insetPadding: insetPadding,
                clipBehavior: clipBehavior,
                shape: shape,
                scrollable: scrollable));

  @override
  Future<T> open(BuildContext context, {RouteArgs args = const {}}) {
    final raw = builder(context, RouteSettings(name: name, arguments: args));
    return AppRoute._result(raw);
  }

  /// In case [result] is success, call [Navigator.pop] with the result, else do nothing.
  static void popResult(BuildContext context, Result result) => result.onSuccess(Navigator.of(context).pop);
}

/// Action of an AlertDialog
///
/// [text] of the action button (can either be a Widget or an object that will be converted [toString])
/// [onAction] callback when the action button is pressed, must return the result that will be dispatched to [onResult]
/// [onResult] handle [Result] of [onAction], by default is [DialogRoute.popResult]
class AlertDialogAction<T> {
  final Object? text;
  final OnAction<T> onAction;
  final OnActionResult<T>? onResult;
  final _ActionId _actionId;

  const AlertDialogAction._(this.text, this.onAction, this.onResult, this._actionId);

  const AlertDialogAction({this.text, required this.onAction, this.onResult = DialogRoute.popResult})
      : _actionId = _ActionId.neutral;

  AlertDialogAction<T> _withAction(_ActionId _actionId) => AlertDialogAction<T>._(text, onAction, onResult, _actionId);

  Widget _build(BuildContext context) {
    final text = this.text ?? _actionId.defaultText(context);
    return TextButton(
        onPressed: () => Result.wrapAsync(() => onAction(context)).then((value) => onResult?.call(context, value)),
        child: text is Widget ? text : Text(text.toString()));
  }
}

enum _ActionId { neutral, negative, positive }

extension _ActionIdExt on _ActionId {
  String defaultText(BuildContext context) {
    final localizations = MaterialLocalizations.of(context);
    switch (this) {
      case _ActionId.neutral:
        return localizations.closeButtonLabel;
      case _ActionId.negative:
        return localizations.cancelButtonLabel;
      case _ActionId.positive:
        return localizations.okButtonLabel;
    }
  }
}

==> 1.0.5-dev.tar.gz: lib/src/route/route_dialog.dart <==
part of '../app_route.dart';

/// Callback when an action is performed, must return the result that will be dispatched to [OnActionResult]
typedef OnAction<T> = FutureOr<T> Function(BuildContext context, RouteArgs outArgs);

/// Handle [Result] of [OnAction]
typedef OnActionResult<T> = void Function(BuildContext context, Result<T> value);

/// Dialog route builder
typedef DialogRouteBuilder = Future Function(BuildContext context, RouteSettings settings);

/// [RouteWidgetBuilder] for AlertDialog that includes [outArgs] parameter that can be updated and used in [OnAction]
typedef AlertRouteWidgetBuilder = Widget Function(BuildContext context, RouteArgs args, RouteArgs outArgs);

/// [AppRoute] of a Dialog
class DialogRoute<T> extends AppRoute<T> {
  final DialogRouteBuilder builder;

  DialogRoute.builder(
    Object name, {
    bool barrierDismissible = true,
    Color? barrierColor = Colors.black54,
    String? barrierLabel,
    bool useSafeArea = true,
    bool useRootNavigator = true,
    required RouteWidgetBuilder builder,
  })  : builder = ((context, settings) => showDialog(
            context: context,
            barrierDismissible: barrierDismissible,
            barrierColor: barrierColor,
            barrierLabel: barrierLabel,
            useSafeArea: useSafeArea,
            useRootNavigator: useRootNavigator,
            routeSettings: settings,
            builder: (context) => builder(context, settings.arguments as RouteArgs))),
        super(name.toString());

  DialogRoute.alertBuilder(
    Object name, {
    Object? title,
    EdgeInsetsGeometry? titlePadding,
    OnAction<T>? onDismiss,
    OnActionResult<T>? onDismissResult = popResult,
    AlertDialogAction<T>? positiveAction,
    AlertDialogAction<T>? negativeAction,
    AlertDialogAction<T>? neutralAction,
    EdgeInsetsGeometry contentPadding = const EdgeInsets.fromLTRB(24.0, 20.0, 24.0, 24.0),
    Color? backgroundColor,
    double? elevation,
    EdgeInsetsGeometry actionsPadding = EdgeInsets.zero,
    EdgeInsetsGeometry? buttonPadding,
    EdgeInsets insetPadding = const EdgeInsets.symmetric(horizontal: 40.0, vertical: 24.0),
    Clip clipBehavior = Clip.none,
    ShapeBorder? shape,
    bool scrollable = false,
    bool barrierDismissible = true,
    Color? barrierColor = Colors.black54,
    String? barrierLabel,
    bool useSafeArea = true,
    bool useRootNavigator = true,
    required AlertRouteWidgetBuilder builder,
  }) : this.builder(name,
            barrierDismissible: barrierDismissible,
            barrierColor: barrierColor,
            barrierLabel: barrierLabel,
            useSafeArea: useSafeArea,
            useRootNavigator: useRootNavigator, builder: (context, args) {
          final outArgs = <String, dynamic>{};
          final alert = AlertDialog(
              title: title == null
                  ? null
                  : title is Widget
                      ? title
                      : Text(title.toString()),
              titlePadding: titlePadding,
              content: builder(context, args, outArgs),
              contentPadding: contentPadding,
              actions: [
                neutralAction?._withAction(_ActionId.neutral),
                negativeAction?._withAction(_ActionId.negative),
                positiveAction?._withAction(_ActionId.positive)
              ].whereNotNull().map((entry) => entry._build(context, outArgs)).toList(growable: false),
              backgroundColor: backgroundColor,
              elevation: elevation,
              actionsPadding: actionsPadding,
              buttonPadding: buttonPadding,
              insetPadding: insetPadding,
              clipBehavior: clipBehavior,
              shape: shape,
              scrollable: scrollable);
          return wrapIfNotNull<Widget, OnAction<T>>(
              item: onDismiss,
              wrap: (child, onDismiss) => WillPopScope(
                  onWillPop: () {
                    Result.wrapAsync(() => onDismiss(context, outArgs))
                        .then((value) => onDismissResult?.call(context, value));
                    return Future.value(false);
                  },
                  child: child),
              child: alert);
        });

  @override
  Future<T> open(BuildContext context, {RouteArgs args = const {}}) {
    final raw = builder(context, RouteSettings(name: name, arguments: args));
    return AppRoute._result(raw);
  }

  /// In case [result] is success, call [Navigator.pop] with the result, else do nothing.
  static void popResult(BuildContext context, Result result) => result.onSuccess(Navigator.of(context).pop);
}

/// Action of an AlertDialog
///
/// [text] of the action button (can either be a Widget or an object that will be converted [toString])
/// [onAction] callback when the action button is pressed, must return the result that will be dispatched to [onResult]
/// [onResult] handle [Result] of [onAction], by default is [DialogRoute.popResult]
class AlertDialogAction<T> {
  final Object? text;
  final OnAction<T> onAction;
  final OnActionResult<T>? onResult;
  final _ActionId _actionId;

  const AlertDialogAction._(this.text, this.onAction, this.onResult, this._actionId);

  const AlertDialogAction({this.text, required this.onAction, this.onResult = DialogRoute.popResult})
      : _actionId = _ActionId.neutral;

  AlertDialogAction<T> _withAction(_ActionId _actionId) => AlertDialogAction<T>._(text, onAction, onResult, _actionId);

  Widget _build(BuildContext context, RouteArgs outArgs) {
    final text = this.text ?? _actionId.defaultText(context);
    return TextButton(
        onPressed: () =>
            Result.wrapAsync(() => onAction(context, outArgs)).then((value) => onResult?.call(context, value)),
        child: text is Widget ? text : Text(text.toString()));
  }
}

enum _ActionId { neutral, negative, positive }

extension _ActionIdExt on _ActionId {
  String defaultText(BuildContext context) {
    final localizations = MaterialLocalizations.of(context);
    switch (this) {
      case _ActionId.neutral:
        return localizations.closeButtonLabel;
      case _ActionId.negative:
        return localizations.cancelButtonLabel;
      case _ActionId.positive:
        return localizations.okButtonLabel;
    }
  }
}
